/* 
 * Copyright  2004
 * Bankenes BetalingsSentral AS (BBS)
 * Organisation Number N- 975 946 231
 * Haavard Martinsens vei 54
 * N- 0045 OSLO
 * Norway
 */

#ifndef STRUCTURES_H
#define STRUCTURES_H

/*	BANKID SDO STRUCTURE */

#define SDO_VERSION		"1.0"
#define SDO_POLICY		"2.16.578.1.16.4.1"
#define SDODATA_TEXT	"text/plain"
#define SDODATA_PDF		"application/pdf"
#define SDODATA_XML		"text/BIDXML"

/* BID_KeyValueElement - an element in the list of key/value-pairs */
typedef struct stBID_AttributeElement {
	char*					szKey;
	char*					szValue;
	struct stBID_AttributeElement*	nextKeyValuePair;
} BID_AttributeElement;


/* BID_Metadata - the data describing this SDOElement */
typedef struct stBID_Metadata {
	char*							szKeywords;
	char*							szDescription;
	struct stBID_AttributeElement*	attributeList;
} BID_Metadata;


/* BID_SignatureList - the list of signature elements */
typedef struct stBID_SignatureElement {
	char*					szDateTime; 
	char*					szSignature;
	unsigned char*					szOCSPResponse; 
	int						iOCSPResponse; 
	char*					szTimestamp;
	struct stBID_SignatureElement*	nextSignatureElement;	
} BID_SignatureElement;


/* BID_SDODatapart - the signed part of a BID_SDOElement */
typedef struct stBID_SDODatapart {
	char*					szPolicy;
	unsigned char*					szHashedData; 
	int						iHashedData; 
	char*					szDatatype;
	struct stBID_SignatureElement*	sigList;
} BID_SDODatapart;


/* BID_SDOElement - each "sub"-SDO in a complete SDO */
typedef struct stBID_SDOElement {
	char*				szBID_SDOVersion;
	struct stBID_SDODatapart*	sdoDatapart;
	char*				szSDOSignature;
	struct stBID_Metadata*			metadata;
	char*				szLocationURL;
	unsigned char*				szStoredData; 
	int					iStoredData; 
	struct stBID_SDOElement*	nextSDOElement;
} BID_SDOElement;


/* BankID_SDO - top level */
typedef struct stBID_SDO {
	struct stBID_SDOElement*	sdoList;
} BID_SDO;

/*	BANKID SDO STRUCTURE */




/*	SEID SDO STRUCTURE */
#define SEIDSDO_VERSION				 "1.0"
#define SEIDSDO_PROFILE				 "SEID-SDO-Basic-V"
#define SEIDSDO_SHA1ALGIDENTIFIER	 "SHA-1"
#define SEIDSDO_SHA256ALGIDENTIFIER	 "SHA-256"
#define SEIDSDO_POLICY				 "2.16.578.1.16.4.1"
#define SEIDSDO_POLICY_URN			 "urn:oid:2.16.578.1.16.4.1"
#define SEIDSDODATA_TEXT			 "text/plain"
#define SEIDSDODATA_PDF				 "application/pdf"
#define SEIDSDODATA_XML				 "text/BIDXML"


/********************* SDODATAPART ****************************************************/

typedef struct st_OCSPValues {
	char*											va_EncapsulatedOCSPValue;
} BID_SEIDSDOOCSPValues;

typedef struct st_RevocationValues {
	struct st_OCSPValues*							OCSPValues;
} BID_SEIDSDORevocationValues;

typedef struct st_UserCertificateAndRevocationData {
	struct st_RevocationValues*						RevocationValues;
} BID_SEIDSDOUserCertificateAndRevocationData;

typedef struct st_HashedData {
	char*											va_DigestMethod;
	char*											va_DigestValue;
} BID_SEIDSDOHashedData;

typedef struct st_SignersDocumentFormat {
	char*											va_MimeType;
} BID_SEIDSDOSignersDocumentFormat;

typedef struct st_SigPolicyId {
	char*											va_Identifier;
} BID_SEIDSDOSigPolicyId;

typedef struct st_SignaturePolicyId {
	struct st_SigPolicyId*							SigPolicyId;
} BID_SEIDSDOSignaturePolicyId;

typedef struct st_SignaturePolicyIdentifier {
	struct st_SignaturePolicyId*					SignaturePolicyId;
} BID_SEIDSDOSignaturePolicyIdentifier;


typedef struct st_CMSSignatureElement {
	char*											va_SDOProfile;
	struct st_SignaturePolicyIdentifier*			SignaturePolicyIdentifier;
	struct st_SignersDocumentFormat*				SignersDocumentFormat;
	struct st_HashedData*							HashedData;
	char*											va_CMSSignature;
	struct st_UserCertificateAndRevocationData*		UserCertificateAndRevocationData;
	
} BID_SEIDSDOCMSSignatureElement;

typedef struct st_SignatureElement {
	struct st_CMSSignatureElement*					CMSSignatureElement;
	struct st_SignatureElement*						nextSignatureElement;
} BID_SEIDSDOSignatureElement;

typedef struct st_SDODataPart {
	struct st_SignatureElement*						SignatureElement;
} BID_SEIDSDODataPart;
/********************* SDODATAPART ****************************************************/


/********************* SDOSEAL ********************************************************/
typedef struct st_SDOSignature {
	struct st_CMSSignatureElement*					CMSSignatureElement;
} BID_SEIDSDOSignature;

typedef struct st_SDOSeal {
	struct st_SDOSignature*							SDOSignature;
} BID_SEIDSDOSeal;
/********************* SDOSEAL ********************************************************/


/********************* METADATA ********************************************************/
typedef struct st_ValuePair{
	char*											va_Name;
	char*											va_Value;
	char*											va_Description;
	struct st_ValuePair*							nextValuePair;
} BID_SEIDSDOValuePair;

typedef struct st_MetaData{
	struct st_ValuePair*							ValuePair;
} BID_SEIDSDOMetaData;
/********************* METADATA ********************************************************/


/********************* SIGNEDOBJECT ****************************************************/
typedef struct st_SignedObject {
	char*											va_SignersDocument;
} BID_SEIDSDOSignedObject;
/********************* SIGNEDOBJECT ****************************************************/


/********************* THE SDO TOP ****************************************************/
typedef struct st_SEIDSDOElement {
	char*											va_SEIDSDOVersion;
	struct st_SDODataPart*							SDODataPart;
	struct st_SDOSeal*								SDOSeal;
	struct st_MetaData*								MetaData;
	struct st_SignedObject*							SignedObject;				
	struct st_SEIDSDOElement*						nextSEIDSDOElement;
	int												va_OldFormat;
} BID_SEIDSDOElement;

typedef struct st_SEIDSDO {
	struct st_SEIDSDOElement*						SEIDSDOElement;
} BID_SEIDSDO;
/********************* THE SDO TOP ****************************************************/

/*	SEID SDO STRUCTURE */


/*	SIGNATURE STRUCTURE */
typedef struct st_Signature {
	char*					pkcs7;
	char*					ocsp;
	struct	st_Signature*	next;
} BID_Signature;
/*	SIGNATURE STRUCTURE */

typedef struct st_SessionInfo {
	int						sessioncontext;
	char*					commonname;
	int						environment;
	char*					grantedpolicies;
	char*					merchantsigncert;
	char*					merchantauthcert;
	struct	st_SessionInfo*	next;
} BID_SessionInfo;
/*	SESSIONINFO STRUCTURE */

#endif
